"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Benchmark = exports.NonZeroExitCodeError = exports.RunIDConflictError = void 0;
const stream_1 = require("stream");
const worker_threads_1 = require("worker_threads");
const parse_json_with_functions_1 = require("./parse-json-with-functions");
const stringify_json_with_functions_1 = require("./stringify-json-with-functions");
class RunIDConflictError extends Error {
    constructor(id) {
        super(`Run with id ${id} already exists.`);
    }
}
exports.RunIDConflictError = RunIDConflictError;
class NonZeroExitCodeError extends Error {
    constructor(code) {
        super(`Worker stopped with non-zero exit code: ${code}`);
    }
}
exports.NonZeroExitCodeError = NonZeroExitCodeError;
class Benchmark extends stream_1.EventEmitter {
    context;
    meta;
    results;
    samples;
    warmup;
    handlers;
    recordPerformance;
    runs;
    constructor({ meta = {}, samples = 10, warmup = true, } = {}) {
        super();
        this.context = {};
        this.handlers = {
            beforeAll: [],
            beforeEach: [],
            afterEach: [],
            afterAll: [],
        };
        this.meta = meta;
        this.meta.title = this.meta.title ?? process.argv[1];
        this.results = new Map();
        this.recordPerformance = false;
        this.runs = new Map();
        this.samples = samples;
        this.warmup = warmup;
    }
    async start() {
        try {
            if (Array.from(this.runs).length === 0)
                return;
            this.emit("start");
            if (this.warmup) {
                this.recordPerformance = false;
                await this.executeRuns();
            }
            this.recordPerformance = true;
            await this.executeRuns();
            this.emit("end");
        }
        catch (error) {
            this.emit("error", error);
        }
    }
    afterAll(func) {
        this.handlers.afterAll.push(func);
    }
    afterEach(func) {
        this.handlers.afterEach.push(func);
    }
    beforeAll(func) {
        this.handlers.beforeAll.push(func);
    }
    beforeEach(func) {
        this.handlers.beforeEach.push(func);
    }
    run(id, file) {
        if (this.runs.has(id)) {
            this.emit("error", new RunIDConflictError(id));
        }
        this.runs.set(id, file);
    }
    buildRuns() {
        return Array.from(this.runs).map(async ([id, file]) => {
            if (!this.results.get(id)) {
                this.results.set(id, []);
            }
            await this.executeHandlers(this.handlers.beforeEach);
            const workerResultRaw = await this.executeRun([id, file]);
            if (this.recordPerformance) {
                const workerResult = (0, parse_json_with_functions_1.parseJSONWithFunctions)(workerResultRaw);
                for (const result of workerResult.results) {
                    await this.executeHandlers(this.handlers.afterEach, result);
                }
                this.results.set(workerResult.id, workerResult.measures);
            }
        });
    }
    executeHandlers(handlers, data) {
        return Promise.all(handlers.map((handler) => handler(this.context, data)));
    }
    executeRun([id, file]) {
        return new Promise((resolve, reject) => {
            const worker = new worker_threads_1.Worker(require.resolve("./worker"), {
                workerData: {
                    id,
                    file,
                    context: (0, stringify_json_with_functions_1.stringifyJSONWithFunctions)(this.context),
                    samples: this.samples,
                },
            });
            worker.on("message", resolve);
            worker.on("error", reject);
            worker.on("exit", (code) => {
                if (code !== 0)
                    reject(new NonZeroExitCodeError(code));
            });
        });
    }
    async executeRuns() {
        await this.executeHandlers(this.handlers.beforeAll);
        const runs = this.buildRuns();
        await Promise.all(runs);
        await this.executeHandlers(this.handlers.afterAll);
    }
}
exports.Benchmark = Benchmark;
